# -*- coding: utf-8 -*-
from aiida.backends.utils import load_dbenv, is_dbenv_loaded
if not is_dbenv_loaded():
    load_dbenv()

import os
import sys
import time
from glob import glob
from os import path
from pymatgen import Structure

from tailcorrection_workchain import TailCorrectionCheck
from tailcorrection_workchain_blocked_pocket import TailCorrectionCheckBlockedPockets
from pathlib import Path 

from aiida.orm import DataFactory
from aiida.orm.data.cif import CifData
from aiida.orm.data.base import Float
from aiida.orm.data.singlefile import SinglefileData
from aiida.work.run import run, submit
import random
import math 

NetworkParameters = DataFactory('zeopp.parameters')
ParameterData = DataFactory('parameter')


NetworkParameters = DataFactory('zeopp.parameters')
ParameterData = DataFactory('parameter')

zeopp_options = {
    "resources": {
        "num_machines": 1,
        "tot_num_mpiprocs": 1,
    },
    "max_wallclock_seconds": 3 * 60 * 60,
    "withmpi": False,
}
zeopp_probe_radius = Float(
    1.86)  # It will create 8 pore blocks for test purpose
zeopp_atomic_radii_file = SinglefileData(file=os.path.abspath(
    "/home/kevin/Documents/uni/EPFL/master_thesis/cof_ml_work/featurization/zeopp.rad"
))  # Radius file for the framework



def get_perpendicular_width(lattice_matrix, cutoff): 
    ax = lattice_matrix[0][0]
    ay = lattice_matrix[0][1]
    az = lattice_matrix[0][2]
    bx = lattice_matrix[1][0]
    by = lattice_matrix[1][1]
    bz = lattice_matrix[1][2]
    cx = lattice_matrix[2][0]
    cy = lattice_matrix[2][1]
    cz = lattice_matrix[2][2]
    
    # calculate vector products of cell vectors
    axb1 = ay * bz - az * by
    axb2 = az * bx - ax * bz
    axb3 = ax * by - ay * bx
    bxc1 = by * cz - bz * cy
    bxc2 = bz * cx - bx * cz
    bxc3 = bx * cy - by * cx
    cxa1 = cy * az - ay * cz
    cxa2 = ax * cz - az * cx
    cxa3 = ay * cx - ax * cy
    
    # calculate volume of cell    
    V = math.fabs(ax * bxc1 + ay * bxc2 + az * bxc3)
    
    # calculate cell perpendicular widths
    perp_width = [0.0] * 3
    perp_width[0] = V / math.sqrt(bxc1**2 + bxc2**2 + bxc3**2)
    perp_width[1] = V / math.sqrt(cxa1**2 + cxa2**2 + cxa3**2)
    perp_width[2] = V / math.sqrt(axb1**2 + axb2**2 + axb3**2)
    
    return perp_width

def get_correct_multiplication_factors(s, cutoff):
    lattice = s.lattice
    perp_width = get_perpendicular_width(lattice.matrix, cutoff)
    multipl_length = [0] * 3
    for k in range(3): 
        multipl_length[k] = int(math.ceil(2 * cutoff / perp_width[k]))
    return multipl_length[0], multipl_length[1], multipl_length[2]


forcefields = ['DREIDINGUFF-TraPPE_TC', 'DREIDINGUFF-TraPPE']
cutoffs = [6.0, 8.0, 10.0, 12.0, 14.0, 16.0, 18.0, 20.0, 22.0, 24.0, 26.0]
path = '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study'
structures = ['/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/16411C2.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/BENXUP_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/17030N2.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/17120N2.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/IBICIH_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/TUDJIM_freeONLY_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/YUJNIB_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/UMCM-1.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/XOJWID_charged_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/EMT.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/18091N2.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/NOCKUM_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/SFS.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/NUTQEZ_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/RIVDEF_clean_min.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/13030N2.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/SFN.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/Mg-MOF-74.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/AFT.cif',
       '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/15030N2.cif']

for cif in structures:
    s = CifData(file=cif)
    s.label = Path(cif).stem
    for ff in forcefields:
        for cutoff in cutoffs:
            print('Preparing {} with cutoff of {}'.format(cif, cutoff))
            spymatgen = Structure.from_file(cif)
            nx, ny, nz = get_correct_multiplication_factors(spymatgen, cutoff)
            unitscells = "{} {} {}".format(nx, ny, nz)
            if cutoff == 20.0:
                raspa_params = ParameterData(
                    dict={
                        "GeneralSettings": {
                            "SimulationType": "MonteCarlo",
                            "CutOff": cutoff,
                            "ChargeMethod": "None",
                            "Forcefield": ff,
                            "UnitCells": unitscells,
                            "ExternalTemperature": 298.0,
                            "ComputeRDF": "yes",
                            "WriteRDFEvery": 1000,
                            'RemoveAtomNumberCodeFromLabel': 'yes',
                        },
                        "Component": [{
                            "MoleculeName": "methane",
                            "MoleculeDefinition": "TraPPE",
                        }],
                    })

            else:
                raspa_params = ParameterData(
                    dict={
                        "GeneralSettings": {
                            "SimulationType": "MonteCarlo",
                            "CutOff": cutoff,
                            "ChargeMethod": "None",
                            "Forcefield": ff,
                            "UnitCells": unitscells,
                            "ExternalTemperature": 298.0,
                             'RemoveAtomNumberCodeFromLabel': 'yes',
                        },
                        "Component": [{
                            "MoleculeName": "methane",
                            "MoleculeDefinition": "TraPPE",
                        }],
                    })

            outputs = submit(
                TailCorrectionCheck,
                structure=s,
                raspa_parameters=raspa_params,
                raspa_code=Code.get_from_string('raspa@fidis'),
                _label='tailcorrection_correct_multiple'
            )

            outputs = submit(
                TailCorrectionCheckBlockedPockets,
                structure=s,
                zeopp_code=Code.get_from_string('zeopp@fidis'),
                _zeopp_options=zeopp_options,
                zeopp_probe_radius=zeopp_probe_radius,
                zeopp_atomic_radii=zeopp_atomic_radii_file,
                raspa_parameters=raspa_params,
                raspa_code=Code.get_from_string('raspa@fidis'),
            )

            time.sleep(5)
