# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function
import os
import time
from aiida.common.example_helpers import test_and_get_code
from aiida.orm import DataFactory

THIS_DIR = os.getcwd()
STRUCTURE_DIR = '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_final/Core-MOF-new'

CifData = DataFactory('cif')
SinglefileData = DataFactory('singlefile')
NetworkParameters = DataFactory('zeopp.parameters')

code = test_and_get_code('zeopp@deneb', expected_code_type='zeopp.network')

d = {
    'res': True, 
    'ha': True,
    'sa': [1.86, 1.86, 100000],
    'volpo': [1.86, 1.86, 100000],
    'chan': 1.86,
}

parameters = NetworkParameters(dict=d)

for s in os.listdir(STRUCTURE_DIR):
    try: 
        structure = CifData(file=os.path.join(STRUCTURE_DIR, s))
    except Exception:
        print("Could not parse cif")
    else: 
        calc = code.new_calc()
        calc.label = "zeopp-core_mofs_for_sampling"
        calc.set_max_wallclock_seconds(2 * 60 * 60)
        calc.set_withmpi(False)
        calc.set_resources({"num_machines": 1})
        calc.use_parameters(parameters)

        structure = CifData(file=os.path.join(STRUCTURE_DIR, s))
        calc.use_structure(structure)

        atomic_radii = SinglefileData(
            file=os.path.abspath(
                "/home/kevin/Documents/uni/EPFL/master_thesis/cof_ml_work/featurization/zeopp.rad"
            ))  #

        calc.use_atomic_radii(atomic_radii)
        calc.store_all()
        calc.submit()
        print("submitted calculation; calc=Calculation(uuid='{}') # ID={}".format(
            calc.uuid, calc.dbnode.pk))
        time.sleep(10)